package scanner

import file.FilePosition
import org.junit.Test

import java.io.InputStream

import org.junit.Assert.assertEquals
import testutil.TestUtilities.createInputStreamFromString

class LispScannerTextTest {

    private fun assertTokenTextMatches(input: String, expectedTextList: Array<String>) {
        val lispScanner = createLispScanner(input)

        for (expectedText in expectedTextList)
            assertEquals(expectedText, lispScanner.nextToken.text)
    }

    private fun assertTokenTextMatches(input: String, expectedText: String) {
        val lispScanner = createLispScanner(input)

        assertEquals(expectedText, lispScanner.nextToken.text)
    }

    private fun createLispScanner(input: String): LispScanner {
        val stringInputStream = createInputStreamFromString(input)

        return LispScanner(stringInputStream, "testFile")
    }

    private fun assertInputFileNameMatches(input: String, expectedInputFileName: String) {
        val stringInputStream = createInputStreamFromString(input)
        val lispScanner = LispScanner(stringInputStream, expectedInputFileName)
        val (fileName) = lispScanner.nextToken.position

        assertEquals(expectedInputFileName, fileName)
    }

    @Test
    fun givenEmptyStream_RecordsCorrectFileName() {
        val input = ""
        val expectedFileName = "testFileName"

        assertInputFileNameMatches(input, expectedFileName)
    }

    @Test
    fun givenParenthesis_RecordsCorrectText() {
        val input = "()"
        val expected = arrayOf("(", ")")

        assertTokenTextMatches(input, expected)
    }

    @Test
    fun givenQuote_RecordsCorrectText() {
        val input = "'"
        val expected = "'"

        assertTokenTextMatches(input, expected)
    }

    @Test
    fun givenEof_ReordsCorrectText() {
        val input = ""
        val expected = "EOF"

        assertTokenTextMatches(input, expected)
    }

    @Test
    fun givenIdentifier_RecordsCorrectText() {
        val input = "identifier"

        assertTokenTextMatches(input, input)
    }

    @Test
    fun givenNumber_RecordsCorrectText() {
        val input = "192837456"

        assertTokenTextMatches(input, input)
    }

    @Test
    fun givenString_RecordsCorrectText() {
        val input = "\"String!!! \n More... \""

        assertTokenTextMatches(input, input)
    }

    @Test
    fun givenNumberFollowedByComment_RecordsCorrectText() {
        val input = "192837456;comment"
        val expected = "192837456"

        assertTokenTextMatches(input, expected)
    }

    @Test
    fun givenIdentifiersWithCommentBetween_RecordsCorrectText() {
        val input = "abc123;comment\nabc222"
        val expected = arrayOf("abc123", "abc222")

        assertTokenTextMatches(input, expected)
    }

    @Test
    fun givenBackTickExpression_RecordsCorrectText() {
        val input = "`(list ,a ,@b)"
        val expected = arrayOf("`", "(", "list", ",", "a", ",", "@", "b", ")")

        assertTokenTextMatches(input, expected)
    }
}
